@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -33,10 +34,10 @@ use Polymake::Struct (
 sub unify_edge_decor {
    my ($name, $decor, $default)=@_;
    my $unified;
-   if (is_container($decor)) {
+   if (is_like_array($decor)) {
       $unified=&unify_decor;
       sub { $unified->(${$_[0]}) }
-   } elsif (is_hash($decor)) {
+   } elsif (is_like_hash($decor)) {
       if (is_code($default)) {
          # assuming that $default was already filtered thru unify_edge_decor,
          # we must avoid repeated dereferencing
@@ -91,7 +92,7 @@ use Polymake::Struct (
 
 sub check_edges {
    my ($name, $edges)=@_;
-   is_array($edges) ? $edges : croak( "$name is not an array" );
+   is_like_array($edges) ? $edges : croak( "$name is not an array" );
 }
 
 sub n_nodes { scalar(@{(shift)->Vertices}) }
