@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_GRAPH_SPRING_EMBEDDER_H
-#define POLYMAKE_GRAPH_SPRING_EMBEDDER_H
+#pragma once
 
 #include "polymake/client.h"
 #include "polymake/Matrix.h"
@@ -109,7 +108,6 @@ public:
 
 } }
 
-#endif // POLYMAKE_GRAPH_SPRING_EMBEDDER_H
 
 // Local Variables:
 // mode:C++
