@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_GRAPH_CONNECTED_H
-#define POLYMAKE_GRAPH_CONNECTED_H
+#pragma once
 
 #include "polymake/graph/graph_iterators.h"
 #include "polymake/IncidenceMatrix.h"
@@ -162,7 +161,6 @@ class generic_of_GraphComponents<GraphRef, polymake::graph::connected_components
    : public GenericSet< GraphComponents<GraphRef, polymake::graph::connected_components_iterator>, Set<Int>, operations::cmp > {};
 }
 
-#endif // POLYMAKE_GRAPH_CONNECTED_H
 
 // Local Variables:
 // mode:C++
