@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -70,8 +71,8 @@ sub attrs2text {
    my $attrs=shift;
    join(", ", map {
       "$_=" . do {
-	 my $val=$attrs->{$_};
-	 is_ARRAY($val) ? '"'.join(",", @$val).'"' : $val
+         my $val = $attrs->{$_};
+         is_array($val) ? '"'.join(",", @$val).'"' : $val
       }
    # We sort the keys to have consistent output for all perl versions (especially >=5.18)
    } sort keys %$attrs)
@@ -141,22 +142,22 @@ sub toString {
    for (my ($n, $end)=(0, $self->Graph->n_nodes); $n<$end; ++$n) {
       %node_attrs=();
       if (defined($node_labels)) {
-	 $node_attrs{label}='"'.$node_labels->($n).'"';
+         $node_attrs{label}='"'.$node_labels->($n).'"';
       }
       if (defined($node_color)) {
-	 process_fill_color(\%node_attrs, $node_color->($n));
+         process_fill_color(\%node_attrs, $node_color->($n));
       }
       if (defined($node_border_color)) {
-	 process_color(\%node_attrs, $node_border_color->($n));
+         process_color(\%node_attrs, $node_border_color->($n));
       }
       if (defined($node_thickness)) {
-	 process_box_thickness(\%node_attrs, $node_thickness->($n));
+         process_box_thickness(\%node_attrs, $node_thickness->($n));
       }
       if (defined($node_border_thickness))  {
-	 process_line_thickness(\%node_attrs, $node_border_thickness->($n));
+         process_line_thickness(\%node_attrs, $node_border_thickness->($n));
       }
       if (defined($node_style)) {
-	 process_style(\%node_attrs, $node_style->($n));
+         process_style(\%node_attrs, $node_style->($n));
       }
       $text.="  n$n" . (keys %node_attrs ? " [" . attrs2text(\%node_attrs) . "];\n" : ";\n");
    }
@@ -164,24 +165,55 @@ sub toString {
    for (my $e=$self->Graph->all_edges; $e; ++$e) {
       %edge_attrs=();
       if (defined($edge_labels)) {
-	 $edge_attrs{label}='"'.$edge_labels->($e).'"';
+         $edge_attrs{label}='"'.$edge_labels->($e).'"';
       }
       if (defined($edge_color)) {
-	 process_color(\%edge_attrs, $edge_color->($e));
+         process_color(\%edge_attrs, $edge_color->($e));
       }
       if (defined($edge_thickness)) {
-	 process_line_thickness(\%edge_attrs, $edge_thickness->($e));
+         process_line_thickness(\%edge_attrs, $edge_thickness->($e));
       }
       if (defined($edge_style)) {
-	 process_style(\%edge_attrs, $edge_style->($e));
+         process_style(\%edge_attrs, $edge_style->($e));
       }
       if (defined($arrows)) {
-	 process_dir(\%edge_attrs, $arrows->($e));
+         process_dir(\%edge_attrs, $arrows->($e));
       }
       my ($s,$t)=$self->reverse_arrows ? reverse(@$e) : @$e;
       $text.="  n$s " . $self->edge_symbol . " n$t" .
              (keys %edge_attrs ? " [" . attrs2text(\%edge_attrs) . "];\n" : ";\n");
    }
+   if (instanceof Visual::Lattice($self->Graph) && $self->Graph->Mode eq "primal") {
+      if(defined($self->Graph->Dims)) {
+         my $dimmap = $self->Graph->Dims;
+         my $rankcommands = "";
+         for(my $map_it = entire($dimmap->get_map()); $map_it; $map_it++) {
+            # If a level is not connected to the next level, we need an invisible edge
+            my @thislevelnodes = @{ $dimmap->nodes_of_rank($$map_it->first)};
+            next if scalar(@thislevelnodes) == 0;
+            my @nextlevelnodes = @{ $dimmap->nodes_of_rank($$map_it->first+1) // []};
+            my $has_edge = 1;
+            if(scalar(@nextlevelnodes)) {
+               $has_edge = 0;
+               for my $tln (@thislevelnodes) {
+                  for my $nln (@nextlevelnodes) {
+                     if($self->Graph->has_edge($tln, $nln)) {
+                        $has_edge = 1; last;
+                     }
+                  }
+                  last if $has_edge;
+               }
+            }
+            if(!$has_edge) {
+               $text.= "n".$nextlevelnodes[0]." -> n".$thislevelnodes[0]." [style=\"invis\"];\n";
+            }
+            # Tell graphviz about nodes of same rank
+            $rankcommands.=("{ rank=same; ".join("; ", map { "n".$_ } @thislevelnodes)." };\n");
+         }
+         $text.=$rankcommands;
+      }
+   }
+
    $text.="}\n";
 }
 
@@ -189,7 +221,7 @@ sub extract_seed {
    my $Graph=shift;
    if (is_object(my $emb=$Graph->Coord)) {
       if (instanceof Visual::GraphEmbedding($emb) && defined (my $seed=$emb->options->{seed})) {
-	 return "; start=$seed";
+         return "; start=$seed";
       }
    }
    "";
@@ -222,5 +254,7 @@ sub addLattice {
 1
 
 # Local Variables:
-# c-basic-offset:3
+# mode: perl
+# cperl-indent-level: 3
+# indent-tabs-mode:nil
 # End:
