@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -225,7 +226,7 @@ sub draw_vertices {
    my $style=$self->source->VertexStyle;
    undef $style unless is_code($style);
    my $color=$self->source->VertexColor;
-   my ($default_color_is_fill, $draw_border);
+   my $default_color_is_fill;
    my $border_color=$self->source->VertexBorderColor;
    my $border_thickness=$self->source->VertexBorderThickness;
    my $draw_border= defined($border_color) || defined($border_thickness);
