@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -73,7 +74,7 @@ sub prep_node_decor : method {
    my ($name, $decor, $default)=@_;
    my $unified= $name =~ /Labels$/ ? &unify_labels : &unify_decor;
    if (is_code($unified)  and
-       !is_container($decor) || @$decor > $self->n_nodes  and
+       !is_like_array($decor) || @$decor > $self->n_nodes  and
        my $renumber=$self->needs_renumbering) {
       sub { $unified->($renumber->[shift]) }
    } else {
