@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2021
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -17,6 +17,8 @@
 
 #include "polymake/client.h"
 #include "polymake/graph/Lattice.h"
+#include "polymake/graph/maximal_chains.h"
+#include "polymake/graph/max_cliques.h"
 
 namespace polymake { namespace graph {
 
@@ -32,8 +34,34 @@ BigObject lattice_permuted_faces(BigObject lattice_obj, const Permutation& perm)
   return static_cast<BigObject>((Lattice<Decoration, SeqType>(lattice_obj)).permuted_faces(perm));
 }
 
+template <typename Decoration>
+Array<Set<Int>> lattice_maximal_chains(BigObject lattice_obj)
+{
+  const Lattice<Decoration> HD(lattice_obj);
+  return maximal_chains(HD, true, true);
+}
+
+template <typename Decoration>
+Graph<Undirected> lattice_comparability_graph(BigObject lattice_obj)
+{
+  const Lattice<Decoration> HD(lattice_obj);
+  const Int d = HD.graph().nodes()-2; // don't count top and bottom
+  const Array<Set<Int>> max_chains = lattice_obj.give("MAXIMAL_CHAINS");
+  Graph<Undirected> CG(d);
+  for (auto c = entire(max_chains); !c.at_end(); ++c) {
+    if (c->size()>1) {
+      for (auto pair = entire(all_subsets_of_k(*c,2)); !pair.at_end(); ++pair) {
+        CG.edge(pair->front()-1, pair->back()-1);
+      }
+    }
+  }
+  return CG;
+}
+
 FunctionTemplate4perl("lattice_dual_faces<Decoration, SeqType>(Lattice<Decoration, SeqType>)");
 FunctionTemplate4perl("lattice_permuted_faces<Decoration, SeqType, Permutation>(Lattice<Decoration,SeqType>, Permutation)");
+FunctionTemplate4perl("lattice_maximal_chains<Decoration>(Lattice<Decoration>)");
+FunctionTemplate4perl("lattice_comparability_graph<Decoration>(Lattice<Decoration>)");
 
 } }
 
