@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_CANONICALIZE_H
-#define POLYMAKE_POLYTOPE_CANONICALIZE_H
+#pragma once
 
 
 #include "polymake/client.h"
@@ -40,4 +39,3 @@ void canonicalize_oriented(Iterator&& it)
 
 } }
 
-#endif
