@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef __POLYMAKE_GALE_TOOLS_H
-#define __POLYMAKE_GALE_TOOLS_H
+#pragma once
 
 namespace polymake { namespace polytope {
 
@@ -54,7 +53,6 @@ Matrix<E> balance(const GenericMatrix<TMatrix, E>& G_arg)
 
 } } // namespaces
 
-#endif // __POLYMAKE_GALE_TOOLS_H
 
 
 // Local Variables:
