@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_IS_REGULAR_H
-#define POLYMAKE_POLYTOPE_IS_REGULAR_H
+#pragma once
 
 #include "polymake/PowerSet.h"
 #include "polymake/Matrix.h"
@@ -138,12 +137,11 @@ secondary_cone_ineq(const GenericMatrix<Matrix, Scalar>& full_dim_verts, const A
          inequs /= new_row(l, full_dim_verts, basis, basis_sign, basis_det);
 
    }
-   return std::pair<const SparseMatrix<Scalar>,const SparseMatrix<Scalar>>(inequs, equats);
+   return std::pair<const SparseMatrix<Scalar>,const SparseMatrix<Scalar>>(remove_zero_rows(inequs), remove_zero_rows(equats));
 }
 
 } }
 
-#endif // POLYMAKE_POLYTOPE_IS_REGULAR_H
 
 // Local Variables:
 // mode:C++
