@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -23,7 +24,7 @@ use Polymake::Struct (
    '@points',
    '@colors',
    '$dim',
-   [ '$different_x_y' => 'new Map<Vector<Float>, ARRAY>' ],
+   [ '$different_x_y' => 'new Map<Vector<Float>, Set<Int>>' ],
    '%different_angles',
    [ '$whites' => 'new Map<Vector<Float>, Int>' ],
    [ '$blacks' => 'new Map<Vector<Float>, Int>' ],
@@ -43,7 +44,7 @@ sub new {
    my $v=0;
    foreach my $gv (@{$self->Polytope->GALE_VERTICES}) {
       my $color=$gv->[0];
-      my $point=$gv->slice(1);
+      my $point=$gv->slice(range_from(1));
       push @{$self->colors}, $color;
       push @{$self->points}, $point;
       if ($color) {
@@ -51,7 +52,7 @@ sub new {
             my $angle=atan2($Transform->elem($v,1), $Transform->elem($v,0));
             push @{$self->different_angles->{$angle}}, $v;
          }
-         push @{$self->different_x_y->{$point}}, $v;
+         $self->different_x_y->{$point} += $v;
          if ($color>0) {
             $self->whites->{$point}=1;
             delete $self->blacks->{$point};
