@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -58,10 +59,10 @@ void make_pentagon(Matrix<QE>& V)
    // The order of reflections is dictated by the root vectors above;
    // the vertices will be in the order given by their convex hull.
    V.row(0) = r1+r2; V(0,0) = QE(1,0,5);  // a point to start reflecting in
-   V.row(1) = reflect(SparseVector<QE>(V.row(0)), r0);  // FIXME: Get rid of the explicit SparseVector<QE>() 
-   V.row(2) = reflect(SparseVector<QE>(V.row(1)), r2);
-   V.row(3) = reflect(SparseVector<QE>(V.row(2)), r4);
-   V.row(4) = reflect(SparseVector<QE>(V.row(3)), r1);
+   V.row(1) = reflect(V.row(0), r0);
+   V.row(2) = reflect(V.row(1), r2);
+   V.row(3) = reflect(V.row(2), r4);
+   V.row(4) = reflect(V.row(3), r1);
 }
 
 void complete_configuration(Matrix<QE>& V)
@@ -107,12 +108,12 @@ void complete_configuration(Matrix<QE>& V)
 
 
    // We replace point 0 by the barycenter of the pentagon
-   for (int i=1; i<5; ++i)
+   for (Int i = 1; i < 5; ++i)
       V.row(0) += V.row(i);
    V.row(0).dehomogenize();
 
    // we translate along the kernel vector to get something of rank 3
-   for (int i=0; i<9; ++i)
+   for (Int i = 0; i < 9; ++i)
       V.row(i) += nv;
 
    // ... and negate the first point. 
@@ -124,7 +125,7 @@ void complete_configuration(Matrix<QE>& V)
    V.row(10) = - V.row(3); 
    V.row(11) = - V.row(4); 
 
-   V = V.minor(All, ~scalar2set(0));
+   V = V.minor(All, range_from(1));
 }
 
 void apply_transform(Matrix<QE>& V)
@@ -135,13 +136,7 @@ void apply_transform(Matrix<QE>& V)
 
    // Since by construction {1,9}, {3,10} and {4,11} already sum to
    // zero, it suffices to balance the rest.
-   Set<int> rowset; 
-   rowset += 0;
-   rowset += 2;
-   rowset += 5;
-   rowset += 6;
-   rowset += 7;
-   rowset += 8;  // c++11 will spare us this
+   const Set<Int> rowset{ 0, 2, 5, 6, 7, 8};
 
    // a basis for the coefficients that make the scaled rows sum to zero:
    const Matrix<QE> M = null_space(T(V.minor(rowset, All))); 
@@ -152,8 +147,8 @@ void apply_transform(Matrix<QE>& V)
       throw std::runtime_error("Couldn't find an all-positive kernel vector");
 
    // rescale the relevant vectors so that the configuration is balanced
-   Entire<Vector<QE> >::const_iterator vit = entire(coeffs);
-   for (Entire<Set<int> >::const_iterator sit = entire(rowset); !sit.at_end(); ++sit, ++vit)
+   auto vit = entire(coeffs);
+   for (auto sit = entire(rowset); !sit.at_end(); ++sit, ++vit)
       V.row(*sit) *= *vit;
    
    // ... and check.
@@ -163,7 +158,7 @@ void apply_transform(Matrix<QE>& V)
 
 } // end anonymous namespace
 
-perl::Object perles_irrational_8_polytope()
+BigObject perles_irrational_8_polytope()
 {
    Matrix<QE> V(12,4);  // The points of the Gale transform
 
@@ -186,17 +181,18 @@ perl::Object perles_irrational_8_polytope()
    W.col(0) = ones_vector<QE>(W.rows());
 
    // done.
-   perl::Object p("Polytope<QuadraticExtension<Rational>>");
+   BigObject p("Polytope<QuadraticExtension<Rational>>",
+               "POINTS", W,
+               "BOUNDED", true,
+               "POINTED", true,
+               "GALE_TRANSFORM", V);
    p.set_description() << "An 8-dimensional polytope with 12 vertices due to Perles that has no rational realization" << endl;
-   p.take("POINTS") << W;
-   p.take("BOUNDED") << true;
-   p.take("POINTED") << true;
-   p.take("GALE_TRANSFORM") << V;
    return p;
 }
 
 UserFunction4perl("# @category Producing a polytope from scratch"
                   "# Create an 8-dimensional polytope without rational realizations due to Perles"
+                  "# See Grünbaum, Convex polytopes, 2nd ed., Springer (2003), p.94f"
                   "# @return Polytope",
                   &perles_irrational_8_polytope, "perles_irrational_8_polytope()");
 
