@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2021
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -54,7 +54,7 @@ UserFunctionTemplate4perl("# @category Optimization"
                           "# If //LP// comes with an attachment 'INTEGER_VARIABLES' (of type Array<Bool>),"
                           "# the output will contain an additional section 'GENERAL',"
                           "# allowing for IP computations in CPLEX."
-                          "# If the polytope is not FEASIBLE, the function will throw a runtime error."
+                          "# If the polytope is already known to be infeasible, the function will throw a runtime error."
                           "# Alternatively one can also provide a //MILP//, instead of a //LP// with 'INTEGER_VARIABLES' attachment."
                           "# @param Polytope P"
                           "# @param LinearProgram LP default value: //P//->LP"
