@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -143,24 +143,21 @@ quotient_space_simplexity_ilp(Int d
       Equations    = (zero_vector<Scalar>(Blocked_Cocircuit_Equations.rows() + Compatibility_Equations.rows()) | (Blocked_Cocircuit_Equations / Compatibility_Equations) ) 
                      / volume_vect;
 
-   BigObject lp("LinearProgram", mlist<Scalar>());
-   lp.attach("INTEGER_VARIABLES") << Array<bool>(n,true);
-   lp.take("LINEAR_OBJECTIVE") << Vector<Scalar>(0|ones_vector<Scalar>(n));
-
-   BigObject q("Polytope", mlist<Scalar>());
-   q.take("FEASIBLE") << true;
-   q.take("INEQUALITIES") << Inequalities;
-   q.take("EQUATIONS") << remove_zero_rows(Equations);
-   q.take("LP") << lp;
+   BigObject q("Polytope", mlist<Scalar>(),
+               "FEASIBLE", true,
+               "INEQUALITIES", Inequalities,
+               "EQUATIONS", remove_zero_rows(Equations));
+   BigObject lp = q.add("LP", "LINEAR_OBJECTIVE", 0 | ones_vector<Scalar>(n));
+   lp.attach("INTEGER_VARIABLES") << Array<bool>(n, true);
 
    const std::string filename = options["filename"];
 
    if (filename.size()) {
       if (filename == "-") {
-         print_lp<Scalar>(q, lp, false, perl::cout);
+         print_lp<Scalar, true>(q, lp, false, perl::cout);
       } else {
          std::ofstream os(filename.c_str());
-         print_lp<Scalar>(q, lp, false, os);
+         print_lp<Scalar, true>(q, lp, false, os);
       }
    }
    return q;
