@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,26 +15,34 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_TOPAZ_GRAPH_H
-#define POLYMAKE_TOPAZ_GRAPH_H
+#pragma once
 
 #include "polymake/Graph.h"
 #include "polymake/FacetList.h"
+#include "polymake/topaz/complex_tools.h"
 
 namespace polymake { namespace topaz {
 
 // Computes the graph of a simplicial complex.
-template <typename Complex> inline
-Graph<> vertex_graph(const Complex& C);
+template <typename Complex>
+Graph<> vertex_graph(const Complex& C)
+{
+   const PowerSet<Int> OSK = k_skeleton(C, 1);
+   const Set<Int> V = accumulate(OSK, operations::add());
+
+   Graph<> G(V.back()+1);
+   for (auto c_it = entire(OSK); !c_it.at_end(); ++c_it)
+      if (c_it->size() == 2)  // is edge
+         G.edge(c_it->front(), c_it->back());
+
+   return G;
+}
 
 // Computes the dual graph of a simplicial complex.
 Graph<> dual_graph(const FacetList& C);
 
 } }
 
-#include "polymake/topaz/graph.tcc"
-
-#endif // POLYMAKE_TOPAZ_GRAPH_H
 
 // Local Variables:
 // mode:C++
