@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_TROPICAL_THOMOG_H
-#define POLYMAKE_TROPICAL_THOMOG_H
+#pragma once
 
 #include "polymake/Rational.h"
 #include "polymake/Matrix.h"
@@ -134,7 +133,6 @@ normalized_first(const GenericVector<VectorTop, TropicalNumber<Addition, Scalar>
 
 } }
 
-#endif
 
 // Local Variables:
 // mode:C++
