@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -20,6 +20,10 @@
 #include "polymake/TropicalNumber.h"
 #include "polymake/Vector.h"
 #include "polymake/Matrix.h"
+/* #include "polymake/GF2.h" */
+#include "polymake/Graph.h"
+#include "polymake/graph/Lattice.h"
+#include "polymake/graph/Decoration.h"
 #include "polymake/IncidenceMatrix.h"
 #include "polymake/tropical/patchwork.h"
 
@@ -27,8 +31,12 @@ namespace polymake { namespace tropical {
 
 FunctionTemplate4perl("real_facets<Addition>(Array<Bool>, Matrix<Int>, Vector<TropicalNumber<Addition>>, Matrix<Rational>, IncidenceMatrix<NonSymmetric>)");
 
+FunctionTemplate4perl("real_phase<Addition>(Array<Bool>, Matrix<Int>, Vector<TropicalNumber<Addition>>, Matrix<Rational>, IncidenceMatrix<NonSymmetric>)");
+
 FunctionTemplate4perl("real_part_realize<Addition>(Matrix<Int>, Vector<TropicalNumber<Addition>>, Matrix<Rational>, IncidenceMatrix<NonSymmetric>, Set<Int>, IncidenceMatrix<NonSymmetric>, String)");
 
+FunctionTemplate4perl("chain_complex_from_dualsub(Array<Bool>, Lattice<BasicDecoration>, Matrix<Rational>)");
+
 } }
 
 // Local Variables:
