@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2020
+#  Copyright (c) 1997-2023
 #  Ewgenij Gawrilow, Michael Joswig, and the polymake team
 #  Technische Universität Berlin, Germany
 #  https://polymake.org
@@ -33,7 +33,7 @@ sub usage {
 }
 
 sub check_bundled {
-   -e "bundled/cdd/external/cdd/lib-src-gmp/cdd.h"
+   -e "bundled/cdd/external/cdd/lib-src/cdd.h"
 }
 
 sub proceed {
@@ -130,7 +130,7 @@ int main() {
                    "The complete error log follows:\n\n$message\n",
                    "Please investigate the reasons and fix the installation.\n";
          } else {
-            my ($ddver,$ddfver) = $message =~ /dd Version ([0-9a-z.]+) .*\nddf Version ([0-9a-z.]+) /;
+            my ($ddver,$ddfver) = $message =~ /dd Version ([0-9a-z.]+).*\nddf Version ([0-9a-z.]+)/;
             if ($ddver ne $ddfver) {
                check_bundled() and !defined($cdd_path) or 
                   die "Your cddlib installation does not seem to contain the floating-point arithmetic version.\n",
@@ -156,9 +156,9 @@ int main() {
 
    if ($UseBundled) {
       die "bundled cdd requested but it cannot be found" 
-         unless (-e "bundled/cdd/external/cdd/lib-src-gmp/cdd.h");
+         unless check_bundled();
       undef $LIBS;
-      $CFLAGS='-I${root}/bundled/cdd/external/cdd/lib-src-gmp';
+      $CFLAGS='-I${buildroot}/staticlib/cdd';
    } else {
       $LIBS="-lcddgmp";
    }
