@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -20,6 +20,12 @@
 #pragma clang diagnostic ignored "-Wshadow"
 #pragma clang diagnostic ignored "-Wzero-as-null-pointer-constant"
 #pragma clang diagnostic ignored "-Wconversion"
+#if !defined(__APPLE__) && __clang_major__ >= 13
+#pragma clang diagnostic ignored "-Wunused-but-set-variable"
+#endif
+#if defined(__APPLE__) && __clang_major__ >= 12
+#pragma clang diagnostic ignored "-Wrange-loop-analysis"
+#endif
 #elif defined(__GNUC__)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wshadow"
@@ -29,9 +35,9 @@
 
 #include <gmpxx.h>
 
-#include "libnormaliz/libnormaliz.h"
-#include "libnormaliz/cone.h"
-#include "libnormaliz/HilbertSeries.h"
+#include <libnormaliz/libnormaliz.h>
+#include <libnormaliz/cone.h>
+#include <libnormaliz/HilbertSeries.h>
 
 #if defined(__clang__)
 #pragma clang diagnostic pop
@@ -104,7 +110,7 @@ std::vector<std::vector<Scalar>> pmMatrix_to_stdvectorvector(const Matrix<FromSc
 // create libnormaliz cone object based on given type
 //  from rays or inequalities
 template <typename Scalar>
-libnormaliz::Cone<Scalar> libnormaliz_create_cone(BigObject c, bool from_ineq, bool compute_facets, bool with_grading)
+libnormaliz::Cone<Scalar> libnormaliz_create_cone(BigObject c, libnormaliz::ConeProperties& todo, bool from_ineq, bool compute_facets, bool with_grading)
 {
   std::map<libnormaliz::InputType, std::vector<std::vector<Scalar>>> inputmap;
   Matrix<Rational> data;
@@ -121,8 +127,14 @@ libnormaliz::Cone<Scalar> libnormaliz_create_cone(BigObject c, bool from_ineq, b
     }
     // lookup dual description if we do not want to compute it
     if (!compute_facets) {
-      if (c.lookup("FACETS | INEQUALITIES") >> data)
+      if (c.lookup("FACETS | INEQUALITIES") >> data) {
         inputmap[libnormaliz::Type::inequalities] = pmMatrix_to_stdvectorvector<Scalar, Rational>(data);
+#if NMZ_RELEASE == 30904
+        // this avoids a bug in normaliz 3.9.4 which triggers a failed assertion
+        // see https://github.com/Normaliz/Normaliz/issues/400
+        todo.set(libnormaliz::ConeProperty::NoCoarseProjection);
+#endif
+      }
       if (c.lookup("LINEAR_SPAN | EQUATIONS") >> data)
         inputmap[libnormaliz::Type::equations] = pmMatrix_to_stdvectorvector<Scalar, Rational>(data);
     }
@@ -152,10 +164,10 @@ RationalFunction<> nmz_convert_HS(const libnormaliz::HilbertSeries& nmzHilb)
 
 template <typename Scalar>
 ListReturn normaliz_compute_with(BigObject c, OptionSet options,
-                                       const libnormaliz::ConeProperties& todo, bool with_grading)
+                                       libnormaliz::ConeProperties& todo, bool with_grading)
 {
   ListReturn result;
-  libnormaliz::Cone<Scalar> nmzCone = libnormaliz_create_cone<Scalar>(c, options["from_facets"], options["facets"], with_grading);
+  libnormaliz::Cone<Scalar> nmzCone = libnormaliz_create_cone<Scalar>(c, todo, options["from_facets"], options["facets"], with_grading);
   // in the first case we have done computation with long already or disabled it explicitly
   // the second case is for non-bundled libnormaliz where the long variant is handled
   // by libnormaliz directly
@@ -245,31 +257,12 @@ ListReturn normaliz_compute(BigObject c, OptionSet options)
 
   const bool with_grading = c.exists("MONOID_GRADING");
 
-#ifdef BUNDLED_LIBNORMALIZ
-  if (!options["skip_long"]) {
-    try {
-      // try with long first
-      return normaliz_compute_with<long>(c, options, todo, with_grading);
-    }
-    catch(const pm::GMP::error& ex) {
-      if (libnormaliz::verbose)
-        cerr << "libnormaliz: error converting coordinates to long, retrying with arbitrary precision" << endl;
-    }
-    catch(const libnormaliz::ArithmeticException& ex) {
-      if (libnormaliz::verbose)
-        cerr << "libnormaliz: arithmetic error detected, retrying with arbitrary precision" << endl;
-    }
-  }
-  return normaliz_compute_with<Integer>(c, options, todo, with_grading);
-#else
   /*
-     * non-bundled installation of libnormaliz doesnt support using our own type
+     * libnormaliz doesnt support using our own type anymore
      * for mpz_class cones libnormaliz will first try to compute with long long,
        thus we do not need do it separately
   */
   return normaliz_compute_with<mpz_class>(c, options, todo, with_grading);
-#endif
-
 }
 
 
@@ -280,30 +273,12 @@ Matrix<Integer> normaliz_compute_lattice(const Matrix<Integer> & V, int threads
    if (threads)
       libnormaliz::set_thread_limit(threads);
 
-#ifdef BUNDLED_LIBNORMALIZ
-   try {
-      // try with long first
-      return normaliz_compute_lattice_with<long>(V);
-   }
-   catch(const pm::GMP::error& ex) {
-      if (libnormaliz::verbose)
-         cerr << "libnormaliz: error converting coordinates to long, retrying with arbitrary precision" << endl;
-      return normaliz_compute_lattice_with<Integer>(V);
-   }
-   catch(const libnormaliz::ArithmeticException& ex) {
-      if (libnormaliz::verbose)
-         cerr << "libnormaliz: arithmetic error detected, retrying with arbitrary precision" << endl;
-      return normaliz_compute_lattice_with<Integer>(V);
-   }
-#else
    /*
-    * non-bundled installation of libnormaliz doesnt support using our own type
+    * libnormaliz doesnt really support using our own type anymore
     * for mpz_class cones libnormaliz will first try to compute with long long,
-    thus we do not need do it separately
+    * thus we do not need do it separately
     */
    return normaliz_compute_lattice_with<mpz_class>(V);
-#endif
-
 }
 
 
