@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -55,7 +56,8 @@ sub proceed {
    }
    if ($ppl_lib) {
       if (-f "$ppl_lib/libppl.$Config::Config{so}") {
-         $LDFLAGS = "-L$ppl_lib -Wl,-rpath,$ppl_lib";
+         $LDFLAGS = "-L$ppl_lib";
+         $LDFLAGS .= " -Wl,-rpath,$ppl_lib" unless $ppl_lib =~ m#^/usr/lib#;
       } elsif ("$ppl_lib/libppl.a") {
 	 die "PPL found at $ppl_path is built without shared libraries.\n";
       } else {
@@ -93,6 +95,10 @@ int main() {
           "Please remember to enable C++ interface and GMP support when configuring the libppl!\n";
    }
 
+   if (defined($Polymake::Configure::GCCversion) && Polymake::Configure::v_cmp($Polymake::Configure::GCCversion, "8.0.0") >= 0) {
+      $CXXFLAGS .= " -Wno-class-memaccess";
+   }
+
    $LIBS="-lppl";
    return "$ppl_version @ ".($ppl_path//"system");
 }
