@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_IDEAL_INTERNAL_SINGULAR_INCLUDE_H
-#define POLYMAKE_IDEAL_INTERNAL_SINGULAR_INCLUDE_H
+#pragma once
 
 #if defined(__clang__)
 #pragma clang diagnostic push
@@ -39,4 +38,3 @@
 #pragma GCC diagnostic pop
 #endif
 
-#endif
