@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2020
+#  Copyright (c) 1997-2023
 #  Ewgenij Gawrilow, Michael Joswig, and the polymake team
 #  Technische Universität Berlin, Germany
 #  https://polymake.org
@@ -69,10 +69,11 @@ sub proceed {
    if(!$hasSCIP){
       # we try with libsoplex-pic.a first (for cmake builds)
       # if this cannot be found we try libsoplex.so (from make SHARED=true) later
-      $LIBS="-lsoplex-pic -lz";
+      $LIBS="-lz";
    } else {
       $LIBS = $Polymake::Bundled::scip::LIBS;
    }
+   $LIBS .= " -lsoplex-pic";
 
 
    # example code from src/example.cpp
@@ -129,8 +130,8 @@ RETRY:
          }
 
          ($version) = $output =~ /SoPlex version ([\d.]+)/;
-         if (Polymake::Configure::v_cmp($version, "2.2.0") < 0) {
-            die "SoPlex version is $version. Minimal required version is 2.2.0\n",
+         if (Polymake::Configure::v_cmp($version, "4.0.0") < 0) {
+            die "SoPlex version is $version. Minimal required version is 4.0.0\n",
                 "Full version string:\n$output\n\n";
          }
          ($gmp) = $output =~ /\[rational: ([-\d\w\s.]+)\]/;
