@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,12 +15,11 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 /** @file SparseVector.h
     @brief Implementation of pm::SparseVector class
  */
 
-#ifndef POLYMAKE_SPARSE_VECTOR_H
-#define POLYMAKE_SPARSE_VECTOR_H
 
 #include "polymake/GenericVector.h"
 #include "polymake/internal/tree_containers.h"
@@ -40,7 +40,7 @@ public:
 template <typename NodeRef>
 struct sparse_vector_index_accessor {
    typedef NodeRef argument_type;
-   typedef const int result_type;
+   typedef Int result_type;
    result_type operator() (NodeRef n) const { return n.key_and_data.first; }
 };
 
@@ -54,16 +54,16 @@ struct sparse_vector_index_accessor {
 template <typename E>
 class SparseVector
    : public modified_tree< SparseVector<E>,
-                           mlist< ContainerTag< AVL::tree< AVL::traits<int, E, operations::cmp> > >,
+                           mlist< ContainerTag< AVL::tree< AVL::traits<Int, E> > >,
                                   OperationTag< pair< BuildUnary<sparse_vector_accessor>,
                                                       BuildUnary<sparse_vector_index_accessor> > > > >
    , public GenericVector<SparseVector<E>, E> {
    typedef modified_tree<SparseVector> base_t;
 protected:
-   typedef AVL::tree< AVL::traits<int, E, operations::cmp> > tree_type;
+   using tree_type = AVL::tree< AVL::traits<Int, E> >;
    struct impl {
       tree_type tree;
-      int d;
+      Int d;
 
       impl() : d(0) {}
       void clear() { d=0; tree.clear(); }
@@ -79,7 +79,7 @@ protected:
    typedef sparse_proxy_base<SparseVector> proxy_base;
 
    template <typename Iterator>
-   void init(Iterator&& src, int dim_arg)
+   void init(Iterator&& src, Int dim_arg)
    {
       data.get()->d=dim_arg;
       data.get()->tree.assign(std::forward<Iterator>(src));
@@ -88,25 +88,25 @@ protected:
 public:
    using typename GenericVector<SparseVector>::generic_type;
 
-   typedef random_access_iterator_tag container_category;
-   typedef sparse_elem_proxy<proxy_base> reference;
+   using container_category = random_access_iterator_tag;
+   using reference = sparse_elem_proxy<proxy_base>;
 
    tree_type& get_container() { return data->tree; }
    const tree_type& get_container() const { return data->tree; }
 
    /// tell the current vector dimension, i.e., the number of non-zero elements (may differ from size)
-   int dim() const { return data->d; }
+   Int dim() const { return data->d; }
 
    /// create as empty
    SparseVector() {}
 
    /// create vector of length n, (implicitly) initialize all elements with 0
-   explicit SparseVector(int dim_arg) { data.get()->d=dim_arg; }
+   explicit SparseVector(Int dim_arg) { data.get()->d=dim_arg; }
 
    /** Create a vector of dimension n, initialize the elements from a data sequence.
-       Iterator can be either indexed, or supply index-value pairs, e.g. std::pair<int,ElementType> or a plain sequence of data items. In the latter case zero elements are filtered out.*/
+       Iterator can be either indexed, or supply index-value pairs, e.g. std::pair<Int, ElementType> or a plain sequence of data items. In the latter case zero elements are filtered out.*/
    template <typename Iterator, bool enabled=construct_sparse_iterator<Iterator, E>::enabled>
-   SparseVector(int dim_arg, Iterator&& src)
+   SparseVector(Int dim_arg, Iterator&& src)
    {
       init(construct_sparse_iterator<Iterator, E>()(std::forward<Iterator>(src), dim_arg), dim_arg);
    }
@@ -119,29 +119,29 @@ public:
    template <typename Vector2>
    SparseVector(const GenericVector<Vector2, E>& v)
    {
-      init(ensure(v.top(), (pure_sparse*)0).begin(), v.dim());
+      init(ensure(v.top(), pure_sparse()).begin(), v.dim());
    }
 
    /// Create a vector as a copy of another vector with a different element type.
    template <typename Vector2, typename E2>
    explicit SparseVector(const GenericVector<Vector2, E2>& v,
-                         typename std::enable_if<can_initialize<E2, E>::value, void**>::type=nullptr)
+                         std::enable_if_t<can_initialize<E2, E>::value, std::nullptr_t> = nullptr)
    {
-      init(make_converting_iterator<E>(ensure(v.top(), (pure_sparse*)0).begin()), v.dim());
+      init(make_converting_iterator<E>(ensure(v.top(), pure_sparse()).begin()), v.dim());
    }
 
    /// Create a vector from a list of values.
    /// Zeroes are filtered out automatically.
-   template <typename E2, typename=typename std::enable_if<can_initialize<E2, E>::value>::type>
+   template <typename E2, typename = std::enable_if_t<can_initialize<E2, E>::value>>
    SparseVector(std::initializer_list<E2> l)
    {
-      init(make_converting_iterator<E>(ensure(l, (pure_sparse*)0).begin()), l.size());
+      init(make_converting_iterator<E>(ensure(l, pure_sparse()).begin()), l.size());
    }
 
    /// Create a vector from a list of (index, value) pairs.
    /// Parameter @a d specifies the dimension.
    /// Indices don't have to come in ascending order; zero values are not detected.
-   SparseVector(int d, std::initializer_list<std::pair<int, E>> l)
+   SparseVector(Int d, std::initializer_list<std::pair<Int, E>> l)
    {
       init(entire(l), d);
    }
@@ -150,7 +150,7 @@ public:
    void clear() { data.apply(shared_clear()); }
 
    /// change the size, initialize appended elements with default constructor
-   void resize(int n)
+   void resize(Int n)
    {
       if (n < data->d) {
          typename base_t::reverse_iterator i=this->rbegin();
@@ -170,7 +170,7 @@ public:
    }
 
    /// random access, may cost O(log(n)) time; \ref vector_performance
-   reference operator[] (int i)
+   reference operator[] (Int i)
    {
       if (POLYMAKE_DEBUG) {
          if (i<0 || i>=dim())
@@ -180,7 +180,7 @@ public:
    }
 
    /// constant random access, may cost O(log(n)) time; \ref vector_performance
-   const E& operator[] (int i) const
+   const E& operator[] (Int i) const
    {
       if (POLYMAKE_DEBUG) {
          if (i<0 || i>=dim())
@@ -191,16 +191,16 @@ public:
 
    /// append a GenericVector
    template <typename Vector2, typename E2,
-             typename=typename std::enable_if<can_initialize<E2, E>::value>::type>
+             typename = std::enable_if_t<can_initialize<E2, E>::value>>
    SparseVector& operator|= (const GenericVector<Vector2, E2>& v)
    {
-      append(v.dim(), make_converting_iterator<E2>(ensure(v.top(), (pure_sparse*)0).begin()));
+      append(v.dim(), make_converting_iterator<E2>(ensure(v.top(), pure_sparse()).begin()));
       return *this;
    }
 
    /// append an element
    template <typename E2,
-             typename=typename std::enable_if<can_initialize<E2, E>::value>::type>
+             typename = std::enable_if_t<can_initialize<E2, E>::value>>
    SparseVector& operator|= (E2&& r)
    {
       if (!is_zero(r)) data->tree.push_back(data->d, std::forward<E2>(r));
@@ -211,10 +211,10 @@ public:
    /// append a list of elements
    /// zeroes are filtered out automatically
    template <typename E2,
-             typename=typename std::enable_if<can_initialize<E2, E>::value>::type>
+             typename = std::enable_if_t<can_initialize<E2, E>::value>>
    SparseVector& operator|= (std::initializer_list<E2> l)
    {
-      append(l.size(), make_converting_iterator<E2>(ensure(l, (pure_sparse*)0).begin()));
+      append(l.size(), make_converting_iterator<E2>(ensure(l, pure_sparse()).begin()));
       return *this;
    }
 
@@ -234,7 +234,7 @@ protected:
       if (data.is_shared()) {
          *this=SparseVector(v);
       } else {
-         data.get()->tree.assign(make_converting_iterator<E>(ensure(v, (pure_sparse*)0).begin()));
+         data.get()->tree.assign(make_converting_iterator<E>(ensure(v, pure_sparse()).begin()));
          data.get()->d=get_dim(v);
       }
    }
@@ -258,10 +258,10 @@ protected:
    }
 
    template <typename Iterator>
-   void append(int added, Iterator&& src)
+   void append(Int added, Iterator&& src)
    {
-      const int d=data->d;
-      tree_type& t=data->tree;
+      const Int d = data->d;
+      tree_type& t = data->tree;
       for (; !src.at_end();  ++src)
          t.push_back(src.index()+d, *src);
       data->d += added;
@@ -273,8 +273,8 @@ protected:
       data->tree.clear();
       if (!is_zero(x)) {
          tree_type& t=data.get()->tree;
-         const int d=data.get()->d;
-         for (int i=0; i<d; ++i) t.push_back(i,x);
+         const Int d = data.get()->d;
+         for (Int i = 0; i < d; ++i) t.push_back(i,x);
       }
    }
 };
@@ -282,36 +282,36 @@ protected:
 template <typename E>
 struct check_container_feature<SparseVector<E>, pure_sparse> : std::true_type {};
 
-template <typename TVector, typename E, typename Permutation> inline
-typename std::enable_if<TVector::is_sparse, SparseVector<E>>::type
+template <typename TVector, typename E, typename Permutation>
+std::enable_if_t<TVector::is_sparse, SparseVector<E>>
 permuted(const GenericVector<TVector, E>& v, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<TVector>::value) {
+   if (POLYMAKE_DEBUG || is_wary<TVector>()) {
       if (v.dim() != perm.size())
          throw std::runtime_error("permuted - dimension mismatch");
    }
    SparseVector<E> result(v.dim());
-   for (auto p=ensure(perm, (cons<end_sensitive,indexed>*)0).begin(); !p.at_end();  ++p) {
+   for (auto p = entire<indexed>(perm); !p.at_end();  ++p) {
       auto e=v.top().find(*p);
       if (!e.at_end()) result.push_back(p.index(), *e);
    }
    return result;
 }
 
-template <typename TVector, typename E, typename Permutation> inline
-typename std::enable_if<TVector::is_sparse, SparseVector<E>>::type
+template <typename TVector, typename E, typename Permutation>
+std::enable_if_t<TVector::is_sparse, SparseVector<E>>
 permuted_inv(const GenericVector<TVector, E>& v, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<TVector>::value) {
+   if (POLYMAKE_DEBUG || is_wary<TVector>()) {
       if (v.dim() != perm.size())
          throw std::runtime_error("permuted_inv - dimension mismatch");
    }
    SparseVector<E> result(v.dim());
-   int pos=0;
-   auto p=perm.begin();
-   for (auto e=entire(v.top());  !e.at_end();  ++e) {
-      std::advance(p, e.index()-pos);
-      pos=e.index();
+   Int pos = 0;
+   auto p = perm.begin();
+   for (auto e = entire(v.top());  !e.at_end();  ++e) {
+      std::advance(p, e.index() - pos);
+      pos = e.index();
       result.insert(*p,*e);
    }
    return result;
@@ -324,11 +324,10 @@ namespace polymake {
 }
 
 namespace std {
-   template <typename E> inline
+   template <typename E>
    void swap(pm::SparseVector<E>& v1, pm::SparseVector<E>& v2) { v1.swap(v2); }
 }
 
-#endif // POLYMAKE_SPARSE_VECTOR_H
 
 // Local Variables:
 // mode:C++
