@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_ITERATOR_ZIPPER_H
-#define POLYMAKE_INTERNAL_ITERATOR_ZIPPER_H
+#pragma once
 
 #include "polymake/internal/comparators_basic_defs.h"
 #include "polymake/ContainerUnion.h"
@@ -393,7 +392,6 @@ public:
 
 } // end namespace pm
 
-#endif // POLYMAKE_INTERNAL_ITERATOR_ZIPPER_H
 
 // Local Variables:
 // mode:C++
