@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_SPARSE_H
-#define POLYMAKE_INTERNAL_SPARSE_H
+#pragma once
 
 #include "polymake/internal/iterator_filters.h"
 #include "polymake/internal/iterator_zipper.h"
@@ -1442,7 +1441,6 @@ void swap_sparse(Container1& c1, Container2& c2)
 
 } // end namespace pm
 
-#endif // POLYMAKE_INTERNAL_SPARSE_H
 
 // Local Variables:
 // mode:C++
