@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
    --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_META_LIST_H
-#define POLYMAKE_META_LIST_H
+#pragma once
 
 #include <type_traits>
 #include "polymake/meta_function.h"
@@ -990,7 +989,6 @@ struct mlist_sort<mlist<>, Less> {
 
 }
 
-#endif // POLYMAKE_META_LIST_H
 
 // Local Variables:
 // mode:C++
