@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -19,7 +20,7 @@ use strict;
 
 my $proto="XS(boot_DynaLoader);\n";
 my $bind=<<'.';
-   newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, __FILE__);
+   newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, "Main.cc");
 .
 
 foreach my $file (@ARGV) {
