@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -27,11 +28,21 @@ declare (%registered_by_dir,     # "AbsPath" => active or disabled Extension
          %registered_by_URI,     # "URI" => active or disabled Extension, maybe replacement
          %refused,               # "URI" => "ignore", "skip", or "stop" : reaction on encountering an unwanted extension in a data file
          %conflicts,             # "URI" => conflicting Extension
+         %disabled               # "AbsPath" => true if can't be configured for the current architecture
         );
 
+declare $loading;                # temporarily set to Extension during loading some rules defined there
+
 # List of obsoleted bundled extensions:
 $refused{'bundled:group'} = 'ignore';
 
+add_settings_callback sub {
+   my ($settings) = @_;
+   $settings->add_item('_extensions::disabled', \%disabled,
+                       "Extensions disabled because of configuration problems",
+                       UserSettings::Item::Flags::hidden + UserSettings::Item::Flags::by_arch + UserSettings::Item::Flags::no_export);
+};
+
 use Polymake::Struct (
    [ new => '$;$$' ],
    [ '$dir' => '#1' ],              # top-level directory
@@ -39,7 +50,7 @@ use Polymake::Struct (
    [ '$version' => 'undef' ],       # version as string
    [ '$version_num' => 'undef' ],   # version as comparable v-string
    [ '$credit' => 'undef' ],        # Credit credit note, if present in metadata
-   [ '$short_name' => '#3'],
+   [ '$short_name' => '#3' ],
    [ '$is_bundled' => 'defined(#3)' ],    # boolean
    [ '$build_dir' => 'undef' ],     # for standalone externsions: where to find architecture-dependent files (configuration, shared modules...)
    [ '$meta_tm' => 'undef' ],       # timestamp of the meta-file
@@ -48,7 +59,6 @@ use Polymake::Struct (
    '@requires_opt',                 # ( Extension ) direct and indirect optional prerequisites
    '@replaces',                     # URIs obsoleted by this Extension
    '@conflicts',                    # URIs conflicting with this Extension
-   [ '$VCS' => 'undef' ],           # version control system for source files
    '$untrusted',                    # TRUE if comes from a writable location, that is, may be under development
    '$configured_at',                # timestamp of the last configuration for the current architecture
 );
@@ -59,7 +69,7 @@ sub new {
    if (-f $self->dir."/polymake.ext") {
       $self->untrusted= -w _ && (!$self->is_bundled || $DeveloperMode);
       $self->meta_tm=(stat _)[9];
-      open my $meta, $self->dir."/polymake.ext"
+      open my $meta, "<:utf8", $self->dir."/polymake.ext"
         or die "can't read extension description ", $self->dir, "/polymake.ext: $!\n";
       local $/;
       local $_=<$meta>;
@@ -83,7 +93,7 @@ sub new {
                 "NAME section is not allowed in the extension description file ", $self->dir, "/polymake.ext\n";
          }
          $short_name =~ s/^\s+//s;  $short_name =~ s/\s+$//s;
-         $self->short_name=$short_name;
+         $self->short_name = $short_name;
       }
 
       if (defined (my $requires=delete $sections{REQUIRE})) {
@@ -106,7 +116,7 @@ sub new {
          $credit =~ s/(?<=\n)(?:[ \t]*\n)*\Z//s;
          $credit =~ s/^[ \t]*/  /gm;
          if ($self->is_bundled) {
-            $self->credit=new Rule::Credit($self->short_name, $credit);
+            $self->credit = new Rule::Credit($self->short_name, $credit);
          } else {
             if (defined($self->short_name)) {
                $credit .= "\n  ".$self->URI."\n";
@@ -131,25 +141,25 @@ sub new {
 
    } elsif (-f $self->dir."/URI") {
       open my $U, $self->dir."/URI";
-      my $URI=<$U>;
+      my $URI = <$U>;
       process_URI($self, $URI);
       if (-w $self->dir) {
+         $self->untrusted = true;
          require Polymake::Core::InteractiveCommands;
          write_initial_description($self);
-         $self->untrusted=1;
       }
 
    } elsif ($self->URI eq "private:") {
-      $self->untrusted=1;
+      $self->untrusted = true;
 
    } else {
       if (defined($self->URI)) {
          die "The extension ", $self->dir, " does not have any URI while ", $self->URI, " was expected\n";
       } else {
-         $self->URI="file://".$self->dir;
+         $self->URI = "file://".$self->dir;
+         $self->untrusted = true;
          require Polymake::Core::InteractiveCommands;
          write_initial_description($self);
-         $self->untrusted=1;
       }
    }
 
@@ -158,13 +168,13 @@ sub new {
 
 # private:
 sub process_URI {
-   my ($self, $URI)=@_;
+   my ($self, $URI) = @_;
    chomp $URI;
    if ($URI =~ s/\#([\d.]+)$//) {
-      $self->version=$1;
-      $self->version_num=eval "v$1";
+      $self->version = $1;
+      $self->version_num = eval "v$1";
    }
-   $self->URI=$URI;
+   $self->URI = $URI;
    delete $refused{$URI};
 }
 ######################################################################################
@@ -175,28 +185,27 @@ sub init {
    {
       # extensions are ordered by inter-dependencies, prerequisites coming first
       foreach my $name (@BundledExts) {
-         my $ext=new(__PACKAGE__, "$InstallTop/bundled/$name", "bundled:$name", $name);
+         my $ext = new(__PACKAGE__, "$InstallTop/bundled/$name", "bundled:$name", $name);
          if ($DeveloperMode && $ext->meta_tm > $ConfigTime) {
             warn_print("meta-file of bundled extension $name has been changed since last configuration;\nPerforming automatic reconfiguration, please be patient...\n");
-            my $build_opt= $ENV{POLYMAKE_BUILD_ROOT} && "--build $ENV{POLYMAKE_BUILD_ROOT}";
-            my $config_log=`cd $InstallTop; $^X support/configure.pl $build_opt 2>&1`;
+            my $build_opt = $ENV{POLYMAKE_BUILD_ROOT} && "--build $ENV{POLYMAKE_BUILD_ROOT}";
+            my $config_log = `cd $InstallTop; $^X support/configure.pl $build_opt 2>&1`;
             if ($?) {
                die "Automatic reconfiguration failed, the complete log including the error diagnostics is shown below.\n",
                   "Please investigate the reasons and rerun the configure script manually, possibly with different options.\n\n",
                   $config_log;
             }
-            my %ConfigFlags=load_config_file("$InstallArch/config.ninja", $InstallTop);
-            $ConfigTime=(stat "$InstallArch/config.ninja")[9];
-            @BundledExts=$ConfigFlags{BundledExts} =~ /(\S+)/g;
-            @active=();
+            my %ConfigFlags = load_config_file("$InstallArch/config.ninja", $InstallTop);
+            $ConfigTime = (stat "$InstallArch/config.ninja")[9];
+            @BundledExts = $ConfigFlags{BundledExts} =~ /(\S+)/g;
+            @active = ();
             redo LOAD;
          }
-         $registered_by_URI{$_}=$ext for $ext->URI, @{$ext->replaces};
-         $registered_by_dir{$ext->dir}=$ext;
-         $ext->VCS=$CoreVCS;
-         $ext->configured_at=max($ConfigTime, $Application::configured_at);
+         $registered_by_URI{$_} = $ext for $ext->URI, @{$ext->replaces};
+         $registered_by_dir{$ext->dir} = $ext;
+         $ext->configured_at = max($ConfigTime, $Application::configured_at);
          my @bad;
-         my @prereqs=map { $registered_by_URI{$_} // do { push @bad, $_; () } } @{$ext->requires};
+         my @prereqs = map { $registered_by_URI{$_} // do { push @bad, $_; () } } @{$ext->requires};
          if (@bad) {
             die "Corrupted configuration of bundled extensions: prerequisites @bad appear after the dependent extension $name or are disabled.\n",
                 "Please investigate and re-run the configure script.\n";
@@ -204,30 +213,30 @@ sub init {
          push @prereqs, grep { defined } map { $registered_by_URI{$_} } @{$ext->requires_opt};
          @{$ext->requires}=uniq(map { ($_, @{$_->requires}) } @prereqs) if @prereqs;
          push @active, $ext;
-         $ext->is_active=1;
+         $ext->is_active = true;
       }
    }
 
    # register the inactive bundled extensions in order to recognize them as prerequisites of other extensions and rulefiles
    foreach my $bundled_dir (glob("$InstallTop/bundled/*")) {
       unless (exists $registered_by_dir{$bundled_dir}) {
-         my ($name)=$bundled_dir =~ $filename_re;
-         my $ext=new(__PACKAGE__, $bundled_dir, "bundled:$name", 1);
-         $registered_by_URI{$ext->URI}=$ext;
+         my ($name) = $bundled_dir =~ $filename_re;
+         my $ext = new(__PACKAGE__, $bundled_dir, "bundled:$name", $name);
+         $registered_by_URI{$ext->URI} = $ext;
       }
    }
 
-   $num_bundled=@active;
+   $num_bundled = @active;
 
    # perform dependency checks on standalone extensions
-   my @pending=map { new(__PACKAGE__, $_) } @User::extensions;
+   my @pending = map { new(__PACKAGE__, $_) } @User::extensions;
    my ($list_updated, @survived, %failed);
 
-   my $rounds=0;
+   my $rounds = 0;
    while (@pending) {
       my @next_round;
       foreach my $ext (@pending) {
-         if (defined (my $other=$registered_by_URI{$ext->URI})) {
+         if (defined(my $other = $registered_by_URI{$ext->URI})) {
             if ($other->URI eq $ext->URI) {
                warn_print( "Extensions ", $other->dir, " and ", $ext->dir, " have identical URI: ", $ext->URI, <<'.' );
 
@@ -242,14 +251,14 @@ and assign distinct URIs.
                            "The first one is therefore removed from your settings.\n",
                            "If this is in error, revise the REPLACE sections in the extension description files" );
             }
-            $list_updated=1;
+            $list_updated = true;
             next;
          }
 
-         my $conflicting=$conflicts{$ext->URI};
+         my $conflicting = $conflicts{$ext->URI};
          unless ($conflicting) {
             foreach (@{$ext->conflicts}) {
-               $conflicting=$registered_by_URI{$_} and last;
+               $conflicting = $registered_by_URI{$_} and last;
             }
          }
          if ($conflicting) {
@@ -258,25 +267,26 @@ and assign distinct URIs.
                         "The first one is therefore removed from your settings.\n",
                         "If this is in error, revise the CONFLICT sections in the extension description files" );
 
-            $failed{$_}="conflicts with other extensions" for $ext->URI, @{$ext->replaces};
-            $list_updated=1;
+            $failed{$_} = "conflicts with other extensions" for $ext->URI, @{$ext->replaces};
+            $list_updated = true;
             next;
          }
 
          my @prereqs;
-         my $satisfied=1;
+         my $satisfied = true;
          for (@{$ext->requires}) {
-            my $prereq_version= s/\#([\d.]+)$// && $1;
-            if (defined (my $prereq=$registered_by_URI{$_})) {
+            my $prereq_version = s/\#([\d.]+)$// && $1;
+            if (defined(my $prereq = $registered_by_URI{$_})) {
                if (length($prereq_version)) {
                   if ($prereq->is_bundled) {
                      warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                                  "imposes a forbidden version requirement on a bundled extension ", $prereq->dir, "\n",
                                  "Please revise the REQUIRE section in the description file ", $ext->dir."/polymake.ext\n",
                                  "and re-import the extension.\n" );
-                     $failed{$_}="wrong dependencies" for $ext->URI, @{$ext->replaces};
-                     $list_updated=1;
-                     $satisfied=0;  last;
+                     $failed{$_} = "wrong dependencies" for $ext->URI, @{$ext->replaces};
+                     $list_updated = true;
+                     $satisfied = false;
+                     last;
 
                   } elsif ($prereq->URI ne $_) {
                      warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
@@ -298,35 +308,38 @@ and assign distinct URIs.
                                  "If this is in error, revise the REQUIRE section in the extension description file\n",
                                  "or try to upgrade the second one to a newer version if available and then re-import the first one" );
 
-                     $failed{$_}="unsatisfied versioned dependencies" for $ext->URI, @{$ext->replaces};
-                     $list_updated=1;
-                     $satisfied=0;  last;
+                     $failed{$_} = "unsatisfied versioned dependencies" for $ext->URI, @{$ext->replaces};
+                     $list_updated = true;
+                     $satisfied = false;
+                     last;
                   }
                }
                push @prereqs, $prereq, @{$prereq->requires};
 
-            } elsif (defined (my $reason=$failed{$_})) {
+            } elsif (defined(my $reason = $failed{$_})) {
                warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                            "depends on another extension $_ which could not be loaded due to $reason.\n",
                            "More details have been reported in the log above.\n",
                            "Re-import this extension when you have fixed the problem in the first place" );
 
-               $failed{$_}="unsatisfied dependencies" for $ext->URI, @{$ext->replaces};
-               $list_updated=1;
-               $satisfied=0;  last;
+               $failed{$_} = "unsatisfied dependencies" for $ext->URI, @{$ext->replaces};
+               $list_updated = true;
+               $satisfied = false;
+               last;
 
             } else {
                push @next_round, $ext;
-               $satisfied=0;  last;
+               $satisfied = false;
+               last;
             }
          }
          next unless $satisfied;
 
-         $registered_by_dir{$ext->dir}=$ext;
-         $registered_by_URI{$ext->URI}=$ext;
+         $registered_by_dir{$ext->dir} = $ext;
+         $registered_by_URI{$ext->URI} = $ext;
          my @old_replaces;
          foreach (@{$ext->replaces}) {
-            if (defined (my $other=$registered_by_URI{$_})) {
+            if (defined(my $other = $registered_by_URI{$_})) {
                if ($other->URI eq $_) {
                   warn_print( "Extension $_ installed at ", $other->dir, "\n",
                               "is declared as replaced by extension ", $ext->URI, " installed at ", $ext->dir, "\n",
@@ -337,13 +350,13 @@ and assign distinct URIs.
                   push @old_replaces, @{$other->replaces};
                   delete @registered_by_URI{@{$other->replaces}};
                   delete_from_list(\@survived, $other);
-                  $list_updated=1;
+                  $list_updated = true;
                }
             }
-            $registered_by_URI{$_}=$ext;
+            $registered_by_URI{$_} = $ext;
          }
 
-         if (my @forgotten_replaces=grep { !exists $registered_by_URI{$_} } @old_replaces) {
+         if (my @forgotten_replaces = grep { !exists $registered_by_URI{$_} } @old_replaces) {
             warn_print( "Extension(s) phased out as announced above provided additional compatibility URIs\n",
                         "which do not occur any more in the current extension ", $ext->dir, "\n",
                         (map { "  $_\n" } @forgotten_replaces),
@@ -353,24 +366,24 @@ and assign distinct URIs.
                         "or consider upgrading dependent extensions to newer versions, if available." );
          }
 
-         $conflicts{$_}=$ext for @{$ext->conflicts};
-         @{$ext->requires}=uniq(@prereqs) if @prereqs;
+         $conflicts{$_} = $ext for @{$ext->conflicts};
+         @{$ext->requires} = uniq(@prereqs) if @prereqs;
          push @survived, $ext;
       }
       last if @next_round == @pending;
       if (++$rounds > 1 && $PrivateDir) {
          warn_print( "Extension order in your settings \@User::extensions is incompatible with the inter-dependencies\n",
                      "between the extensions, they will be automatically reordered.\n",
-                     "Please revise the results in $PrivateDir/prefer.pl after finishing the running polymake session." );
-         $list_updated=1;
+                     "Please revise the results in $PrivateDir/settings after this polymake session has finished." );
+         $list_updated = true;
       }
-      @pending=@next_round;
+      @pending = @next_round;
    }
 
    if (@pending) {
-      my %would_provide=map { ($_ => 1) } map { $_->URI, @{$_->replaces} } @pending;
+      my %would_provide = map { ($_ => true) } map { $_->URI, @{$_->replaces} } @pending;
       foreach my $ext (@pending) {
-         if (my @unknown=grep { !$would_provide{$_} } @{$ext->requires}) {
+         if (my @unknown = grep { !$would_provide{$_} } @{$ext->requires}) {
             warn_print( "Extension ", $ext->URI, " installed at ", $ext->dir, "\n",
                         "has unresolved prerequisite(s)\n",
                         ( map { "  $_\n" } @unknown ),
@@ -387,18 +400,17 @@ and assign distinct URIs.
                         "and re-import the needed extensions in proper order induced by justified dependencies.\n" );
          }
       }
-      $list_updated=1;
+      $list_updated = true;
    }
 
    if ($list_updated) {
-      @User::extensions=grep { exists $registered_by_dir{$_} } @User::extensions;
-      $Prefs->custom->set('@extensions');
+      @User::extensions = grep { exists $registered_by_dir{$_} } @User::extensions;
    }
 
    # filter out extensions disabled for this architecture
 
    foreach my $ext (@survived) {
-      if ($ext->is_active = !$User::disabled_extensions{$ext->dir}) {
+      if ($ext->is_active = !$disabled{$ext->dir}) {
          foreach my $prereq (@{$ext->requires}) {
             unless ($prereq->is_active) {
                if ($prereq->is_bundled) {
@@ -411,7 +423,8 @@ and assign distinct URIs.
                               "which has been disabled for the current architecture $Arch\n",
                               "because of failed build configuration or further dependencies.\n" );
                }
-               $ext->is_active=0;  last;
+               $ext->is_active = false;
+               last;
             }
          }
          unless ($ext->is_active &&= $ext->configure) {
@@ -421,7 +434,7 @@ and assign distinct URIs.
                         "  reconfigure_extension(\"", $ext->dir, "\");" );
          }
          unless ($ext->is_active) {
-            $Prefs->custom->set('%disabled_extensions', $ext->dir, 1);
+            $disabled{$ext->dir} = true;
          }
       }
       if ($ext->is_active) {
@@ -429,23 +442,22 @@ and assign distinct URIs.
       }
    }
 
-   if (my @obsolete=grep { !exists $registered_by_dir{$_} } keys %User::disabled_extensions) {
-      delete @User::disabled_extensions{@obsolete};
-      $Prefs->custom->set('%disabled_extensions');
+   if (my @obsolete = grep { !exists $registered_by_dir{$_} } keys %disabled) {
+      delete @disabled{@obsolete};
    }
 }
 #######################################################################################
 sub configure {
-   my ($self, @options)=@_;
-   my $ext_dir=$self->dir;
-   my $ext_build_dir=$ext_dir;
+   my ($self, @options) = @_;
+   my $ext_dir = $self->dir;
+   my $ext_build_dir = $ext_dir;
 
    if ($ext_build_dir =~ s{^\Q${InstallTop}\E(?=/ext/)}{$InstallArch}o) {
-      $self->build_dir=$ext_build_dir;
+      $self->build_dir = $ext_build_dir;
       if (@options == 1 && $options[0] eq "--help") {
          warn_print( "Extension $ext_dir is already built and installed, it does not need any further configuration." );
       } elsif (-f "$ext_build_dir/config.ninja") {
-         $self->configured_at=max((stat _)[9], $Application::configured_at);
+         $self->configured_at = max((stat _)[9], $Application::configured_at);
          @options and warn_print( "Extension $ext_dir is already built and installed, ignoring configuration options @options." );
          1
       } else {
@@ -461,8 +473,8 @@ For the meanwhile, the extension will stay disabled for architecture $Arch.
       set_build_dir($self);
       unless (@options) {
          if (-f (my $build_file=$self->build_dir."/build.ninja")) {
-            my $conf_tm=(stat _)[9];
-            $self->configured_at=max($conf_tm, $Application::configured_at);
+            my $conf_tm = (stat _)[9];
+            $self->configured_at = max($conf_tm, $Application::configured_at);
             if ($conf_tm >= $self->meta_tm and open my $bf, $build_file) {
                local $_;
                while (<$bf>) {
@@ -490,21 +502,21 @@ For the meanwhile, the extension will stay disabled for architecture $Arch.
 }
 
 sub set_build_dir {
-   my ($self)=@_;
+   my ($self) = @_;
    # workspace polymake and workspace extension should use the same build directory names
    if ($InstallArch =~ m{/build(?:\.[^/]+)?/\w+$}) {
-      $self->build_dir=$self->dir.$&;
+      $self->build_dir = $self->dir.$&;
    } else {
-      $self->build_dir=$self->dir."/build/".($ENV{POLYMAKE_BUILD_MODE} || "Opt");
+      $self->build_dir = $self->dir."/build/".($ENV{POLYMAKE_BUILD_MODE} || "Opt");
    }
 }
 ########################################################################################
 sub lookup {
-   (undef, my $what)=@_;
+   (undef, my $what) = @_;
    my $by_dir;
    unless ($what =~ s{^($id_re)$}{bundled:$1}o) {
       replace_special_paths($what);
-      $by_dir=-d $what and $what=Cwd::abs_path($what);
+      $by_dir = -d $what and $what = Cwd::abs_path($what);
    }
    if ($by_dir) {
       $registered_by_dir{$what} || croak( "Extension at location $what is not registered" )
@@ -516,7 +528,7 @@ sub lookup {
 sub app_dir { $_[0]->dir."/apps/".$_[1]->name }
 
 sub versioned_URI {
-   my $self=shift;
+   my $self = shift;
    $self->URI . (defined($self->version) && "#".$self->version)
 }
 #####################################################################################
@@ -524,41 +536,63 @@ sub versioned_URI {
 # Looks for an extension specified by its URL, imports it if necessary.
 # @return the Extension object in the successful case
 # @return undef if not found or couldn't be loaded
-# @return 0 if the user chose to ignore the request
+# @return false if the user chose to ignore the request
 # Throws an exception if $mandatory and no Extension loaded.
 
 sub provide {
-   my ($pkg, $URI, $mandatory)=@_;
-   if (defined (my $ext=$registered_by_URI{$URI})) {
+   my ($pkg, $URI, $mandatory) = @_;
+   if (defined (my $ext = $registered_by_URI{$URI})) {
       if ($ext->is_active) {
-         $ext;
-      } elsif ($mandatory) {
-         die "Extension $URI is not configured for the current architecture $Arch.\n",
-             "Activate it by polymake shell command\n",
-             "  reconfigure_extension(\"$URI\");\n",
-             "Then try to reload your data.\n";
+         $ext
       } else {
-         warn_print( "Extension $URI is not configured for the current architecture $Arch.\n",
-                     "The properties defined in this extension will be missing in your object.\n",
-                     "If you need them, activate the extension by polymake shell command\n",
-                     "  reconfigure_extension(\"$URI\");\n",
-                     "then reload the object from the file backup copy." );
-         undef;
+         my $short_name = $ext->short_name;
+         my $text1 = $ext->is_bundled ? <<"..bundled.." : <<"..standalone..";
+Bundled extenstion $name is not configured for the current architecture $Arch.
+If you have built polymake from the source code, rerun the configuration script,
+find the explanation of problems in build/bundled.log and fix them,
+then rebuild polymake.
+..bundled..
+Extension $URI is not configured for the current architecture $Arch.
+Activate it with polymake shell command
+  reconfigure_extension("$URI");
+..standalone..
+         if ($mandatory) {
+            die $text1, <<".";
+
+Until then the data file can't be loaded.
+.
+         } else {
+            warn_print( $text1, <<"." );
+
+Until then properties defined in this extension will be kept as "undecoded"
+attachments in your object.  However, they will be preserved in the data file.
+.
+            undef
+         }
       }
 
+   } elsif ($refused{$URI} eq "ignore") {
+      # presumably it's a bundled extension integrated into the core system,
+      # or we already asked about it: ignore it
+      false
+
    } elsif ($URI =~ m|^file://| and -d (my $ext_dir=$')) {
       require Polymake::Core::InteractiveCommands;
       eval { add($pkg, $ext_dir) } or do {
          if ($mandatory) {
             die "Error importing extension from $ext_dir:\n$@";
          } else {
-            warn_print( "Extension at $ext_dir could not be loaded:\n$@",
-                        "The properties defined in this extension will be missing in your object.\n",
-                        "If you need them, investigate the reasons of the failure,\n",
-                        "import the extension with polymake shell commmand\n",
-                        "  import_extension(\"$ext_dir\");\n",
-                        "then reload the object from the file backup copy." );
-            undef;
+            warn_print( <<"." );
+Extension at $ext_dir could not be loaded:
+$@
+Investigate the reasons of the failure and import the extension
+with polymake shell commmand
+  import_extension("$ext_dir");
+
+Until then properties defined in this extension will not be visible in your object,
+although they will be preserved in the data file.
+.
+            undef
          }
       }
 
@@ -566,52 +600,57 @@ sub provide {
       if ($mandatory) {
          die "Extension $URI is required for loading the data.\n";
       } else {
-         warn_print( "Extension $URI is not known.\n",
-                     "The properties defined in this extension will be missing in your object.\n" );
-         undef;
+         warn_print( <<"." );
+Extension $URI is not known.
+Properties defined in this extension will be moved temporarily into "undecoded" attachments,
+although they will be preserved in the data file.
+.
+         undef
       }
 
-   } elsif (!exists $refused{$URI} && defined ($ext=&locate_unknown)) {
-      $ext;
+   } elsif (!exists $refused{$URI} && defined($ext = &locate_unknown)) {
+      $ext
 
    } elsif ($refused{$URI} eq "ignore") {
-      0;
+      false
 
    } elsif ($refused{$URI} eq "stop" || $mandatory) {
       die "Extension $URI is required for loading the data.\n";
 
    } else {
-      undef;
+      undef
    }
 }
 #####################################################################################
 # private:
 sub activate {
-   my ($self)=@_;
-   local $Application::extension=$self;
-   $self->is_active=1;
+   my ($self) = @_;
+   $self->is_active = true;
    foreach my $app_dir (glob($self->dir."/apps/*")) {
       $app_dir =~ $filename_re;
-      if (defined (my $app=lookup Application($1))) {
-         $app->load_extension($app_dir);
+      if (defined (my $app = lookup Application($1))) {
+         $app->load_extension($self, $app_dir);
       }
    }
    push @active, $self;
 }
 #####################################################################################
-sub get_source_VCS {
-   my $self=shift;
-   $self->VCS ||= do {
-      if ($self->is_bundled || index($self->dir, "$InstallTop/ext/")==0) {
-         die "Can't modify an installed bundled extension\n";
-      } elsif (!-w $self->dir) {
-         die "You don't have permission to create or change files in ", $self->dir, "\n";
-      } else {
-         require Polymake::SourceVersionControl;
-         new SourceVersionControl($self->dir);
-      }
+sub get_source_tree {
+   my ($self) = @_;
+   if ($self->untrusted) {
+      require Polymake::SourceTree;
+      new SourceTree($self->is_bundled ? $InstallTop : $self->dir);
+   } elsif ($self->is_bundled || index($self->dir, "$InstallTop/ext/") == 0) {
+      die "Can't modify an installed bundled extension\n";
+   } else {
+      die "You don't have permission to create or change files in ", $self->dir, "\n";
    }
 }
+###############################################################################################
+sub list_prerequisite_extensions {
+   my ($self) = @_;
+   join(" ", map { $_->is_bundled ? $_->short_name : $_->dir } @{$self->requires});
+}
 #####################################################################################
 package Polymake::Core::Extension::Credit;
 
@@ -620,13 +659,13 @@ use Polymake::Struct (
 );
 
 sub display {
-   my ($self)=@_;
+   my ($self) = @_;
    dbg_print( "used extension ", $self->product, "\n", $self->text, "\n" );
-   $self->shown=1;
+   $self->shown = 1;
 }
 
 sub toFileString {
-   my ($self)=@_;
+   my ($self) = @_;
    $self->file_string //= do {
       my ($copyright)= $self->text =~ /(copyright\b.*)/im;
       (defined($copyright) ? "\n$copyright\n" : $self->text) . $self->product . "\n"
