@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -21,8 +22,13 @@ use vars qw( %special_imports );
 $special_imports{"namespaces.pm"}=1;
 
 $INC{"namespaces.pm"}=$INC{"Polymake/Namespaces.pm"};
+$INC{"namespaces/AnonLvalue.pm"}=$INC{"Polymake/Namespaces.pm"};
 $INC{"namespaces/Params.pm"}=$INC{"Polymake/Namespaces.pm"};
-$INC{"namespaces/Package.pm"}=$INC{"Polymake/Namespaces.pm"};
 *Polymake::Namespaces::import=\&import;
 
+sub set_autolookup {
+   my $pkg=caller;
+   Polymake::define_function($pkg, ".AUTOLOOKUP", $_[0]);
+}
+
 1
