@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -69,24 +70,21 @@ sub new {
 }
 
 sub create_testcases {
-   my ($self)=@_;
-   state $testpkg="TutorialTestPkg000000";
+   my ($self) = @_;
+   state $testpkg = "TutorialTestPkg000000";
    ++$testpkg;
 
-   my $file_name = $self->source_file;
-   my $json = '';
-   open my $in, "<:encoding(utf8)", $file_name
-      or die("Error opening $file_name: $!");
-   while (my $line = <$in>) {
-      $json .= $line;
-   }
-   close $in or die $!;
+   my $json = do {
+     local $/;
+     open my $in, "<", $self->source_file
+       or die("Can't read ", $self->source_file, ": $!\n");
+     <$in>
+   };
 
-   my $p = JSON->new->decode($json);
+   my $p = decode_json($json);
    my $name = $self->name;
    my $app = $User::default_application;
-   my $body= <<"---";
-$Polymake::Core::warn_options; 
+   my $body = Core::RuleFilter::warn_options() . <<"---";
 package Polymake::$testpkg;
 use application '$app';  declare +auto;
 include "common::jupyter.rules";
@@ -94,7 +92,7 @@ include "common::jupyter.rules";
 ---
 
    $self->group->env->start_timers;
-   my $cnt=0;
+   my $cnt = 0;
    foreach (@{${$p}{"cells"}}) {
       if (${$_}{"cell_type"} eq "code") { # iterate all code cells
 	 # each cell might introduce independent local effects
@@ -103,7 +101,7 @@ include "common::jupyter.rules";
 
          foreach (@{${$_}{"source"}}) {
 	    s{^ \s* application \s* ($anon_quoted_re) \s*;\s* $}
-	     {use application $1;  declare +auto;\n}xmg;
+	     {use application $1;\n}xmg;
 
             $body .= enforce_nl($_);  # add a trailing newline after the command if not present (for ease of debugging)
          }
@@ -136,30 +134,28 @@ use Polymake::Struct (
    [ '$source_file' => '#3' ],
 );
 
-sub new {
-   my $self=&Case::new;
-   local $disable_viewers = 1;
+sub run_code {
+   my ($self) = @_;
+   local $disable_viewers = true;
+
    my $tdir = new TempChangeDir($self->subgroup->temp_dir);
    # files created in tutorials via save(), save_data() etc. must be automatically destroyed
    # script() must be exempt from this transformation because it refers to existing scripts
-   local_sub(\&replace_special_paths, sub {
-		if ((caller(1))[3] ne "Polymake::User::script") {
-		   foreach (@_) {
-		      s{^~(?=/|$)}{ $self->subgroup->temp_dir }e
-		        or
-		      s{^(?=[^/])}{ $self->subgroup->temp_dir."/" }e;
-		   }
-		}
-	     });
-   before_run($self);
+   local ref *replace_special_paths = sub {
+      if ((caller(1))[3] ne "Polymake::User::script") {
+	 foreach (@_) {
+	    s{^~(?=/|$)}{ $self->subgroup->temp_dir }e
+	      or
+	    s{^(?=[^/])}{ $self->subgroup->temp_dir."/" }e;
+	 }
+      }
+   };
+
    eval $self->body;
    if ($@) {
-      $self->gotten_error=neutralized_ERROR();
-      $@="";
+      $self->gotten_error = neutralized_ERROR();
+      $@ = "";
    }
-   after_run($self);
-   close $self->handle;
-   $self;
 }
 
 sub execute {
@@ -169,7 +165,6 @@ sub execute {
                       "     got: EXCEPTION: ".$self->gotten_error;
       return 0;
    }
-
    1
 }
 
