@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2020
+#  Copyright (c) 1997-2023
 #  Ewgenij Gawrilow, Michael Joswig, and the polymake team
 #  Technische Universität Berlin, Germany
 #  https://polymake.org
@@ -25,6 +25,27 @@ if you already have an up-to-date perl interpreter somewhere else,
 you can specify its location on the command line:
 
 ./configure PERL=/path/to/my/new/perl [other options ...]
+.
+      exit(1);
+   } elsif ($] >= 5.040) {
+      print STDERR <<".";
+*************
+*** ERROR ***
+*************
+
+polymake has not been checkced for compatibility with perl 5.40 or newer;
+your perl interpreter says it is $].
+
+If you already have another (older) perl interpreter somewhere else, you can
+specify its location on the command line:
+
+./configure PERL=/path/to/my/new/perl [other options ...]
+
+You can install a custom perl version in your home directory using perlbrew
+(https://perlbrew.pl). Make sure to append '-Duseshrplib' to the install
+command if you want to use libpolymake (e.g. for the jupyter interface) and to
+install all required perl modules which are listed after running configure
+again with the new perl.
 .
       exit(1);
    }
@@ -86,6 +107,7 @@ $Polymake::DeveloperMode = -d "$root/testscenarios";
 
 my $BuildDir = $options{build} ? "build.$options{build}" : "build";
 my $perlxpath = "perlx/$Config::Config{version}/$Config::Config{archname}";
+my $perlextraflags = "";
 
 load_enabled_bundled_extensions();
 parse_command_line(\@ARGV, $repeating_config);
@@ -108,8 +130,9 @@ locate_gmp_and_mpfr_libraries();
 locate_boost_headers();
 locate_permlib();
 
-# there is no independent package for TOSimplex, always pick the bundled headers
-$ExternalHeaders .= " TOSimplex";
+# there is no independent package for TOSimplex and Minimball, always pick the bundled headers
+$ExternalHeaders .= " TOSimplex Miniball";
+
 
 if ($options{prereq} ne ".none.") {
    check_prerequisite_perl_packages();
@@ -757,7 +780,7 @@ Please investigate and reconfigure.
            $cppver >= 201305 and
                defined($CLANGversion) and v_cmp($CLANGversion, "3.5") < 0
           ) {
-      # C++ standard older than C++14 wont work (C++1y from clang 3.4 is also ok)
+      # C++ standard older than C++14 won't work (C++1y from clang 3.4 is also ok)
       die "C++ standard version ($cppver) too old, C++14 or later is required. Please omit any '-std=' options.\n";
    }
 
@@ -845,11 +868,18 @@ If you have renamed the main fink program, please specify the top directory: --w
 #####################################################
 sub check_brew {
    $BrewBase=$options{brew};
-   if ( $BrewBase eq "default" ) {
+   if ( $BrewBase eq "default" || $BrewBase eq "/usr/local" ) {
       $BrewBase = '/usr/local';
       unless (-f "$BrewBase/bin/brew") {
          die "brew installation corrupt: $BrewBase/bin/brew not found\n";
       }
+   } elsif( $BrewBase eq "bottle" ) {
+      $ADDITIONAL_PERL_INCLUDES = "";
+      my @perl_paths = split(/:/, "$ENV{PERL5LIB}");
+      foreach (@perl_paths) {
+         $ADDITIONAL_PERL_INCLUDES .= " -I$_";
+      }
+      $BrewBase = '#{HOMEBREW_PREFIX}';
    } else {
       if (-d $BrewBase) {
          unless (-f "$BrewBase/bin/brew") {
@@ -903,12 +933,10 @@ sub determine_architecture {
    print "determining architecture ... ";
    if ($^O eq "darwin") {
       if ( !defined($FinkBase) && !defined($BrewBase) ) {
-         if (defined $Arch) {
-            if ($Arch ne "i386" && $Arch ne "x86_64") {
-               die "Invalid architecture $Arch for Mac OS: allowed values are i386 and x86_64.\n";
-            }
-         } else {
-            $Arch = "x86_64";
+         $Arch //= `uname -m`;
+         chomp $Arch;
+         if ($Arch ne "arm64" && $Arch ne "i386" && $Arch ne "x86_64") {
+            die "Invalid architecture $Arch for Mac OS: allowed values are i386, x86_64 and arm64.\n";
          }
       }
       $ARCHFLAGS= $NeedsArchFlag ? "-arch $Arch" : "";
@@ -962,6 +990,9 @@ sub collect_compiler_specific_options {
       if (v_cmp($GCCversion, "10.0.0") >= 0) {
          $CXXFLAGS .= " -Wno-array-bounds";
       }
+      if (v_cmp($GCCversion, "11.0.0") >= 0) {
+         $CXXFLAGS .= " -Wno-maybe-uninitialized -Wno-free-nonheap-object";
+      }
 
    } elsif (defined($ICCversion)) {
       $CsharedFLAGS="-fPIC";
@@ -1212,6 +1243,65 @@ The complete error log follows:
    }
 }
 
+sub check_gmpxx {
+   # check GMP C++ bindings
+   my $build_error=build_test_program(<<'---', LIBS => "-lgmpxx -lgmp");
+#include <cstddef>
+#include <gmpxx.h>
+int main() {
+   mpz_class z(7);
+   mpz_class y(z-z);
+   return y.get_si();
+}
+---
+   if ($?==0) {
+      my $run_error = run_test_program();
+      if ($?) {
+         die "Could not run a test program linked to the C++ version of the GNU Multiprecision Library (GMP).\n",
+             "Probably the shared library libgmpxx.$Config::Config{dlext} is missing or of an incompatible machine type:\n$run_error\n";
+      }
+   } else {
+      die "Could not compile a test program checking for C++ bindings of the GNU Multiprecision Library (GMP).\n",
+          "The most probable reasons are that a gmpxx package is missing, lacking the developer's subpackage \n",
+          "or GMP was configured without C++ support (--enable-cxx).\n",
+          "Please refer to the installation instructions at $Wiki/howto/install.\n",
+          "The complete error log follows:\n", $build_error;
+   }
+}
+
+sub check_gmpxx_ostream {
+   my $extraflags = "";
+   # check GMP C++ library
+OSTREAM_CHECK:
+   my $build_error=build_test_program(<<'---', CXXFLAGS => "$extraflags", LIBS => "$ARCHFLAGS -lgmpxx -lgmp");
+#include <cstddef>
+#include <gmpxx.h>
+#include <iostream>
+
+#ifdef PM_MIXED_OSTREAM
+std::ostream& operator<< (std::ostream &s, mpq_srcptr r) {
+   return s << 1;
+}
+#endif
+int main() {
+   mpq_class x(7,3);
+   std::cout << x << std::endl;
+   return 0;
+}
+---
+      my $run_error = run_test_program() if $? == 0;
+      if ($?!=0) {
+         if ($extraflags) {
+            die "Could not build and run a test program checking the C++ library used by the GNU Multiprecision Library (GMP):\nbuild: $build_error\nrun:$run_error\n";
+         } else {
+            $extraflags = " -DPM_MIXED_OSTREAM";
+            goto OSTREAM_CHECK;
+         }
+      } elsif ($extraflags) {
+         $CXXFLAGS .= $extraflags;
+      }
+}
+
 ##########################################################
 sub locate_boost_headers {
    my $boost_path=$options{boost};
@@ -1348,7 +1438,8 @@ Otherwise, reconfigure and reinstall your perl.
          $_ =~ s/(?:^|\s+)-arch\s+\w+//g for $perlcflags, $perlldflags; 
       }
    }
-   my $build_error=build_test_program(<<'---', CXXFLAGS => (defined($CLANGversion) && "-Wno-reserved-user-defined-literal ").$perlcflags, LDFLAGS => "$ARCHFLAGS $perlldflags" );
+RETRYLIBPERL:
+   my $build_error=build_test_program(<<'---', CXXFLAGS => (defined($CLANGversion) && "-Wno-reserved-user-defined-literal ")."$perlcflags $perlextraflags", LDFLAGS => "$ARCHFLAGS $perlldflags" );
 #include <EXTERN.h>
 #include <perl.h>
 
@@ -1387,6 +1478,14 @@ perl-devel or libperl-dev.  Please look for such a package and install it.
 ---
       }
    } else {
+      if (defined($GCCversion) && $perlextraflags !~ /-D_Thread_local/ && $build_error =~ /error:.+_Thread_local.+does not name a type/s) {
+         # The PERL_THREAD_LOCAL macro might be set to _Thread_local (since perl 5.35.5).
+         # This is defined in the C11 standard and supported by all relevant gcc versions.
+         # But g++ (gcc in C++ mode) does not accept _Thread_local so we define that to thread_local instead.
+         # clang and clang++ both accept _Thread_local.
+         $perlextraflags .= " -D_Thread_local=thread_local";
+         goto RETRYLIBPERL;
+      }
       $error = <<"---";
 Could not compile a test program for the libperl.$Config::Config{so} shared library.
 The build error is as follows:
@@ -1546,6 +1645,9 @@ sub configure_enabled_bundled_extensions {
                print "failed";
                bundled_ext_error_msg($ext, $@);
                $enable=0;
+            } elsif ($result =~ /^disabled/) {
+               $enable=0;
+               print $result;
             } else {
                print "ok", $result ? " ($result)":"";
             }
@@ -1753,7 +1855,7 @@ sub write_perl_specific_configuration_file {
 
    print $conf <<"---";
 PERL=$Config::Config{perlpath}
-CXXglueFLAGS=$includePerlCore$Config::Config{archlibexp}/CORE $Config::Config{ccflags} -DPerlVersion=$PerlVersion $no_warn
+CXXglueFLAGS=$includePerlCore$Config::Config{archlibexp}/CORE $Config::Config{ccflags} $perlextraflags -DPerlVersion=$PerlVersion $no_warn
 LIBperlFLAGS=-L$Config::Config{archlib}/CORE -lperl $Config::Config{ccdlflags}
 ExtUtils_xsubpp=$xsubpp
 ExtUtils_typemap=$typemap
